package com.app.portabledeviceapplication;

import java.util.ArrayList;

import android.content.Context;
import android.graphics.Bitmap;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class ChatRowAdapter extends BaseAdapter{
	private Context context;
	private int textViewResourceId;
	private final ArrayList<String> messageList;
	private final ArrayList<Bitmap> messagePictureList;
	private final ArrayList<String> senderList;
	private final ArrayList<String> timeList;
	private final ArrayList<Boolean> read;
	
	final int THUMBNAIL_SIZE = 128;
	/**
	 * Constructor
	 * @param context Context from class where constructor is made
	 * @param textViewResourceId ID of row-layout, found like this; R.id.(name of layout)
	 * @param messageList List of strings, MessageList, from class where constructor is made
	 * @param pictureList List of Bitmaps, pictureList, from class where constructor is made
	 * @param senderList List of strings, senderList, from class where constructor is made
	 * @param timeList List of strings, timeList, from class where constructor is made
	 * @param readList List of booleans, readList, from class where constructor is made
	 */
	public ChatRowAdapter(Context context, int textViewResourceId, ArrayList<String> messageList,  ArrayList<Bitmap> pictureList, ArrayList<String> senderList, ArrayList<String> timeList, ArrayList<Boolean> readList) {
		super();

	    this.textViewResourceId = textViewResourceId;
	    this.messageList = messageList; 
	    this.messagePictureList = pictureList; 
	    this.senderList = senderList; 
	    this.timeList = timeList; 
	    this.read = readList; 
	    this.context = context;
	}
	
	/**
	 * Function that inflates the layout of the listView
	 * @param position Position of current list position
	 * @param convertView The finished view
	 */
	@Override
	public View getView(int position, View convertView, ViewGroup parent){
		View rowView = convertView;
		ViewHolder holder;
		String sender = senderList.get(position);
		
		if (convertView == null) {
	    	LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			rowView = inflater.inflate(textViewResourceId, parent, false);
			
			holder = new ViewHolder();
			holder.label = (TextView)rowView.findViewById(R.id.label);
			holder.message=(TextView)rowView.findViewById(R.id.message);
			holder.time = (TextView)rowView.findViewById(R.id.textTime);
			holder.check = (ImageView)rowView.findViewById(R.id.checkmark);
			holder.RL = (RelativeLayout)rowView.findViewById(R.id.userAndMessage);
			holder.picture = (ImageView)rowView.findViewById(R.id.messagePicture);
			rowView.setTag(holder);
		}else{
			holder = (ViewHolder)rowView.getTag();
		}
		//Set name of sender, message and time of "sent" of "received"
		holder.label.setText(senderList.get(position));
	    holder.message.setText(messageList.get(position));
	    holder.time.setText(timeList.get(position));
	    // Set background picture and gravity
	    if(sender.equals("Operator")){
	    	if (messageList.get(position)!= null) {
	    		holder.RL.setBackgroundResource(R.drawable.sent);
	    		((RelativeLayout) rowView).setGravity(Gravity.RIGHT);
	    		}
	    	}else{
	    		if (messageList.get(position)!= null) {
	    			holder.RL.setBackgroundResource(R.drawable.received);
	    			((RelativeLayout) rowView).setGravity(Gravity.LEFT);
	    		}
	    	}
	    // Set checkmark, when read message
	    if(read.get(position) == true && sender=="Operator"){
	    	holder.check.setImageResource(R.drawable.check);
	    }else{
	    	holder.check.setImageResource(0);
	    }
	    //Inflate ImageView if message is containing message
	    if((messagePictureList.get(position))!=null){
	    	Bitmap thumbnail = Bitmap.createScaledBitmap(messagePictureList.get(position), THUMBNAIL_SIZE, THUMBNAIL_SIZE, false);
	    	holder.picture.setImageBitmap(thumbnail);
	    }else{
	    	holder.picture.setImageBitmap(null);
	    }
		return rowView;
	}
	
	/**
	 * ViewHolder class, implemented to decrease runtime when inflating layout
	 */
	static class ViewHolder{
		RelativeLayout RL; 
		TextView label;
		TextView message;
		TextView time;
		ImageView check;
		ImageView picture;
		
	}
	
	/** Needed for BaseAdapter **/
	@Override
	public int getCount() {
	    return messageList.size();
	}
	
	/** Needed for BaseAdapter **/
	@Override
	public Object getItem(int position) {
	    return messageList.get(position);
	}
	
	/** Needed for BaseAdapter **/
	@Override
	public long getItemId(int position) {
	    return position;
	}

}
