package com.app.portabledeviceapplication;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Log;
import android.view.Menu;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.EditText;
import android.widget.ListView;



public class MainActivity extends Activity {
	
	private Socket socket;
	boolean run = false; 
	public String msg;
	
	private static final int SERVERPORT = 2810;
	private static final String SERVER_IP = "192.168.10.230";
	
	private static final int SELECT_PICTURE = 0; 
	private static final int SELECT_PHRASE = 1;
	private static final int THUMBNAIL_SIZE = 64;
	
	private PrintWriter printWriter; 
	private BufferedReader bufferedReader;
	
	private ArrayList<String> messageList = new ArrayList<String>();
	private ArrayList<String> senderList = new ArrayList<String>();
	private ArrayList<String> timeList = new ArrayList<String>();
	private ArrayList<Boolean> readList = new ArrayList<Boolean>();
	private ArrayList<Bitmap> pictureMessageList = new ArrayList<Bitmap>();
	private ArrayList<Uri> pictureAddresses = new ArrayList<Uri>();
	private ArrayList<String> IDmessageList = new ArrayList<String>();
	private ArrayList<String> IDsenderList = new ArrayList<String>();
	private ArrayList<String> IDtimeList = new ArrayList<String>();
	private ArrayList<Boolean> IDreadList = new ArrayList<Boolean>();
	private ArrayList<Bitmap> IDpictureMessageList = new ArrayList<Bitmap>();
	private ArrayList<Uri> IDpictureAddresses = new ArrayList<Uri>();
	
	public final static String MESSAGE_TO_DISPLAY = "com.app.portabledeviceapplication.MESSAGE";
	public final static String ID_MESSAGE_ARRAY = "com.app.portabledeviceapplication.ID_MESSAGE_ARRAY";
	public final static String ID_PICTURE_ARRAY = "com.app.portabledeviceapplication.ID_PICTURE_ARRAY";
	public final static String ID_PICTURE_ADDRESSES_ARRAY = "com.app.portabledeviceapplication.ID_PICTURE_ADDRESSES_ARRAY";
	public final static String ID_TIME_ARRAY = "com.app.portabledeviceapplication.ID_TIME_ARRAY";
	public final static String ID_SENDER_ARRAY = "com.app.portabledeviceapplication.ID_SENDER_ARRAY";
	public final static String ID_READ_ARRAY = "com.app.portabledeviceapplication.ID_READ_ARRAY";
	
	private ChatRowAdapter chatAdapter;
	private IDSorter sorter;
	private PictureHandling pictureHandler;
	private SaveLocally saveChat; 
	private Context context = this;
	private ListView listView;
	private EditText et;
	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		listView = (ListView)findViewById(R.id.chatList);
		/* variable to be able to use other classes*/
		chatAdapter = new ChatRowAdapter(this,R.layout.chat_row_layout, messageList, pictureMessageList, senderList, timeList, readList);
		sorter = new IDSorter(this);
		pictureHandler = new PictureHandling(this);
		saveChat = new SaveLocally(this);
		/* check if there is messages stored, that is to be loaded */
		saveChat.checkExternalMedia();
		if(saveChat.sizeOfSDFile("messages.txt")!=0){
			saveChat.readChatFromSDCard(messageList, pictureMessageList, pictureAddresses, senderList, timeList, readList);
		}
		
		listView.setAdapter(chatAdapter);
		listView.setFocusableInTouchMode(true);
		listView.requestFocus();
		/* Adding items to id-lists based on the currently pressed list item*/
		listView.setOnItemClickListener(new OnItemClickListener(){
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id){
				getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
				String content = messageList.get(position);
				sorter.clearIDLists(IDmessageList, IDsenderList, IDtimeList, IDreadList, IDpictureMessageList, IDpictureAddresses);
				if(content.contains("#")){
					String[] ID = content.split(" ");
					for(int j =0; j< ID.length; j++){
						if(ID[j].startsWith("#")){
							sorter.addToArrayBasedOnID(ID[j], messageList, senderList, timeList, readList, pictureMessageList, pictureAddresses,  IDmessageList, IDsenderList, IDtimeList, IDreadList, IDpictureMessageList, IDpictureAddresses);
						}
					}
					Intent intent = new Intent(context, DisplayIDSortedMessagesActivity.class);
					intent.putStringArrayListExtra(ID_MESSAGE_ARRAY, IDmessageList);
					intent.putStringArrayListExtra(ID_SENDER_ARRAY, IDsenderList);
					intent.putStringArrayListExtra(ID_TIME_ARRAY, IDtimeList);
					intent.putExtra(ID_READ_ARRAY, IDreadList);
					intent.putExtra(ID_PICTURE_ARRAY, IDpictureMessageList);
					intent.putExtra(ID_PICTURE_ADDRESSES_ARRAY, IDpictureAddresses);
					startActivity(intent); 
				}else{
					Intent intent = new Intent(context, DisplayMessageActivity.class);
					intent.putExtra(MESSAGE_TO_DISPLAY, content);
					startActivity(intent);
				}
			}
		});
		/* Displaying picture in a bigger view, when holding the list item */
		listView.setOnItemLongClickListener(new OnItemLongClickListener(){
			@Override
			public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {
				if(pictureMessageList.get(position)!=null){
					Intent intent = new Intent();
					intent.setAction(Intent.ACTION_VIEW);
					intent.setDataAndType(pictureAddresses.get(position), "image/*");
					intent.putExtra(messageList.get(position), pictureMessageList.get(position));
					startActivity(intent);
				}
				return true;
			}
		});
		/* Making the camera button clickable*/
		et = (EditText) findViewById(R.id.EditText);
		et.setOnTouchListener(new OnTouchListener(){
			@Override
			public boolean onTouch(View view, MotionEvent event){
				//final int DRAWABLE_LEFT = 0;
	            //final int DRAWABLE_TOP = 1;
	            final int DRAWABLE_RIGHT = 2;
	            //final int DRAWABLE_BOTTOM = 3;
	            
	            if(event.getAction()==MotionEvent.ACTION_UP){
	            	if((event.getX()+80) >= (et.getRight() - et.getCompoundDrawables()[DRAWABLE_RIGHT].getBounds().width())) {
	            		Intent intent = new Intent(context, SavedCategoriesActivity.class);
	            		startActivityForResult(intent,SELECT_PHRASE);
	                }
	            }
				return false; 
			}
		});
		
		chatAdapter.notifyDataSetChanged();
		// Starting ClientThread
		new Thread(new ClientThread()).start();
	}
	/**
	 * Method triggered when pressing the "send"-button
	 * Sends content of EditText to server
	 * @param view
	 */
	public void onSendClick(View view) {
		try {
			et = (EditText) findViewById(R.id.EditText);
			String str = et.getText().toString();
			printWriter = new PrintWriter(new BufferedWriter(
					new OutputStreamWriter(socket.getOutputStream())),
					true);
			printWriter.println(str);
			addToArrays(str, "Operator", false, true, null, null);
			chatAdapter.notifyDataSetChanged();
			et.setText("");
			
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Method triggered when pressing the Camera button
	 * Loading intent to take or use saved picture
	 * @param view
	 */
	public void onCameraClick(View view){
		Intent choose_picture = new Intent(); 
		choose_picture.setType("image/*");
		choose_picture.setAction(Intent.ACTION_GET_CONTENT);
		
		Intent take_picture = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
		String title ="Select picture from phone or take a new one"; 
		Intent chooserIntent = Intent.createChooser(choose_picture, title);
		chooserIntent.putExtra(Intent.EXTRA_INITIAL_INTENTS, new Intent[]{take_picture});
		
		startActivityForResult(chooserIntent, SELECT_PICTURE);
	}
	
	/**
	 * Method triggered when returning from child-activities
	 */
	public void onActivityResult(int requestCode, int resultCode, Intent data){
		super.onActivityResult(requestCode, resultCode, data);
		if(resultCode == RESULT_OK){
			if(requestCode==SELECT_PHRASE){
				//When returning from Saved Category/Content activities
				String returnContent;
				returnContent = data.getStringExtra(SavedCategoriesActivity.TO_MAIN);
				EditText et = (EditText) findViewById(R.id.EditText);
				et.setText(returnContent);
			}if(requestCode==SELECT_PICTURE){
				//When returning with picture
				Uri filePathUri = data.getData();
				Bitmap selected = pictureHandler.getPath(filePathUri);
				int orientation = pictureHandler.getOrientation(filePathUri);
				Matrix matrix = new Matrix();
				EditText et = (EditText) findViewById(R.id.EditText);
			
				BitmapFactory.Options options = new BitmapFactory.Options();
				options.inJustDecodeBounds = true;
				//Spinning picture if it have horizontal orientation in stead of vertical
				if(orientation == 90 || orientation == 270){
					matrix.postRotate(90);
				}else{
					matrix.postRotate(0);
				}
				
				//Making thumbnail of picture
				Bitmap thumbnail = Bitmap.createScaledBitmap(selected, THUMBNAIL_SIZE, THUMBNAIL_SIZE, false);
				Bitmap rotated = Bitmap.createBitmap(thumbnail, 0, 0, thumbnail.getHeight(), thumbnail.getWidth(), matrix, true);
				String test = et.getText().toString();
				
				//Sending picture
				try {
					printWriter = new PrintWriter(new BufferedWriter(
							new OutputStreamWriter(socket.getOutputStream())),
							true);
					printWriter.print(thumbnail);
					addToArrays(test, "Operator", false, true, rotated, filePathUri);
					chatAdapter.notifyDataSetChanged();
					et.setText("");
				
				} catch (UnknownHostException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}else{}
		}
		
	}
	
	/**
	 *ClientThread that listens for messages from server
	 */
	class ClientThread implements Runnable {
		@Override
		public void run() {
			run = true; 
			try{
				InetAddress serverAddr = InetAddress.getByName(SERVER_IP);
				socket = new Socket(serverAddr, SERVERPORT);
				try{
					bufferedReader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
					while(run){
						msg = bufferedReader.readLine().toString();	
						String read = "true";
						if(msg!=null){
							if(msg.equals(read)){for(int i = 0; i<readList.size(); i++){
								if(readList.get(i).equals(false)){
									timeList.set(i, getNewTime());
								}
								readList.set(i, true);
								}
							}else{
								addToArrays(msg, "Control Room", true, false, null, null);
								}
							MainActivity.this.runOnUiThread(new Runnable(){
								@Override
								public void run(){
									chatAdapter.notifyDataSetChanged();
									}
								});
					}
				}
				Log.e("RESPONSE FROM SERVER", "S:received message: '"+msg+"'");
				}catch(Exception e){
					e.printStackTrace();}
				}catch(UnknownHostException e1){
					e1.printStackTrace();
				}catch(IOException e1){
					e1.printStackTrace();
					}
			}
		}
	/**
	 * Adding new messages to arrays that are displayed in the listView
	 * @param msg text message
	 * @param snd sender
	 * @param read if the message is read or not
	 * @param sent If it sent from the portable unit
	 * @param bmp Picture
	 * @param uri filpath of picture
	 */
	public void addToArrays(String msg, String snd, boolean read, boolean sent, Bitmap bmp, Uri uri){
		messageList.add(msg);
		senderList.add(snd);
		String total; 
		if(sent == true){
			String sender = "Sent: ";
			String time = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss", java.util.Locale.getDefault()).format(Calendar.getInstance().getTime());
			total = sender + time; 
		}else{
			String sender = "Received: ";
			String time = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss", java.util.Locale.getDefault()).format(Calendar.getInstance().getTime());
			total = sender + time; 
		}
		timeList.add(total);
		readList.add(read);
		pictureMessageList.add(bmp);
		pictureAddresses.add(uri);
		
		StringBuilder toSDString = new StringBuilder();
		toSDString.append(msg);
		toSDString.append("_");
		toSDString.append(snd);
		toSDString.append("_");
		toSDString.append(total);
		toSDString.append("_");
		toSDString.append(read);
		toSDString.append("_");
		if(uri!=null){
			toSDString.append(uri.toString());
		}else{
			toSDString.append("null");
		}
		saveChat.writeToSDFile(toSDString.toString(), "messages.txt");
	}
	/**
	 * Returns new time when messages are changed from "sent" to "read"
	 */
	public String getNewTime(){
		String sender = "Read: ";
		String time = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss", java.util.Locale.getDefault()).format(Calendar.getInstance().getTime());
		String total = sender + time; 
		return total; 
	}

	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

}
