package com.app.portabledeviceapplication;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.net.Uri;
import android.os.Environment;

public class SaveLocally {
	Context context;
	final int THUMBNAIL_SIZE = 64;
	PictureHandling pictureHandler; 

	public SaveLocally(Context context) {
		
		this.context = context; 
	}
	/**
	 * Check if media is writable and available
	 */
	@SuppressWarnings("unused")
	public void checkExternalMedia() {
		boolean mExternalStorageAvailable = false;
	    boolean mExternalStorageWriteable = false;
	    String state = Environment.getExternalStorageState();
	    if (Environment.MEDIA_MOUNTED.equals(state)) {
	    	// Can read and write the media
	    	mExternalStorageAvailable = mExternalStorageWriteable = true;
	    	} else if (Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
	    		// Can only read the media
	    		mExternalStorageAvailable = true;
	            mExternalStorageWriteable = false;
	        } else {
	            // Can't read or write
	            mExternalStorageAvailable = mExternalStorageWriteable = false;
	        }
	    }
	/**
     * Method to write ascii text characters to file on SD card. Note that you
     * must add a WRITE_EXTERNAL_STORAGE permission to the manifest file or this
     * method will throw a FileNotFound Exception because you won't have write
     * permission.
     */
	public void writeToSDFile(String text, String filename) {
		// Find the root of the external storage.
		File root = android.os.Environment.getExternalStorageDirectory();
		// See http://stackoverflow.com/questions/3551821/android-write-to-sd-card-folder
		File dir = new File(root.getAbsolutePath() + "/download");
		dir.mkdirs();
		File file = new File(dir, filename);
		try {
			FileOutputStream f = new FileOutputStream(file, true);
			PrintWriter pw = new PrintWriter(f);
			pw.println(text);
			pw.flush();
			pw.close();
			f.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
					}
		}
	
	/**
	 * Adding all categories from raw file over to SD-file
	 * @param categoryList ArrayList<String> with all categories
	 */
	public void addAllCategoriesToSD(ArrayList<String> categoryList){
		for(int i = 0; i<categoryList.size()-1; i++){
			StringBuilder categoryToSD = new StringBuilder();
			categoryToSD.append(i);
			categoryToSD.append("_");
			categoryToSD.append(categoryList.get(i));
			writeToSDFile(categoryToSD.toString(), "SavedPhrases.txt");
		}
	}
	/**
	 * Add all category content from raw-file to SD-card file
	 * @param contentLists ArrayList<ArrayList<String>> with all category content
	 */
	public void addAllCategoryContentToSD(ArrayList<ArrayList<String>> contentLists){
		for(int i= 0; i<contentLists.size(); i++){
			for(int j=0; j<contentLists.get(i).size()-1; j++){
				StringBuilder contentToSD = new StringBuilder();
				contentToSD.append(i);
				contentToSD.append("_");
				contentToSD.append(j);
				contentToSD.append("_");
				contentToSD.append(contentLists.get(i).get(j));
				writeToSDFile(contentToSD.toString(), "SavedPhrases.txt");
			}
		}
	}
	/**
	 * Method to read in a text file placed on the local memory, 
	 * which can be saved when application is loaded over and over again 
	 * @param categoryList
	 * @param contentLists
	 * @param filename
	 */
	public void readPhraseFromSDCard(ArrayList<String> categoryList, ArrayList<ArrayList<String>> contentLists, String filename){
		File root = android.os.Environment.getExternalStorageDirectory();
		File dir = new File(root.getAbsolutePath() + "/download");
		dir.mkdirs();
		File file = new File(dir, filename);
		
		StringBuilder text = new StringBuilder();
		try{
			BufferedReader br = new BufferedReader(new FileReader(file));
			String line;
			
			while((line = br.readLine())!=null){
				int count_number = 0;
				text.append(line);
	    		text.append('\n');
	    		
				String[] content = line.split("_");
				for(int j =0; j< content.length; j++){
					if(Character.isDigit(content[j].charAt(0))){
						count_number = count_number +1; 
					}
				}
				if(count_number == 1){	
					int position = Integer.parseInt(content[0]);
					
					categoryList.add((position), content[1]);
					contentLists.add((position), new ArrayList<String>());
					contentLists.get(position).add("Add new content");
					
				}
				if(count_number == 2){
					int categoryPosition = Integer.parseInt(content[0]);
					int size = contentLists.get(categoryPosition).size();
					contentLists.get(categoryPosition).add((size-1), content[2]);
				}
	    	}
	    	 br.close();
	    	 }catch(IOException e){
	    		 e.printStackTrace();
	    		 
	    	 }
	    	 
	}
	
	/**
	 * Function to read sent messages from txt-file placed in local memmory.
	 * Parameters is list where the result from the read is to be stored
	 * @param messageList
	 * @param pictureMessageList
	 * @param pictureAddresses
	 * @param senderList
	 * @param timeList
	 * @param readList
	 */
	public void readChatFromSDCard(ArrayList<String> messageList, ArrayList<Bitmap> pictureMessageList, ArrayList<Uri> pictureAddresses, ArrayList<String> senderList, ArrayList<String> timeList, ArrayList<Boolean> readList){
		File root = android.os.Environment.getExternalStorageDirectory();
		File dir = new File(root.getAbsolutePath() + "/download");
		dir.mkdirs();
		File file = new File(dir, "messages.txt");
		pictureHandler = new PictureHandling(context);
		
		StringBuilder text = new StringBuilder();
		try{
			BufferedReader br = new BufferedReader(new FileReader(file));
			String line;
			
			while((line = br.readLine())!=null){
				text.append(line);
	    		text.append('\n');
	    		
				String[] content = line.split("_");
				messageList.add(content[0]);
				senderList.add(content[1]);
				timeList.add(content[2]);
				readList.add(Boolean.parseBoolean(content[3]));
				
				
				if(!content[4].equals("null")){
					Uri filepath= Uri.parse(content[4]);
					pictureAddresses.add(filepath);
					
					Bitmap selected = pictureHandler.getPath(filepath);

					int orientation = pictureHandler.getOrientation(filepath);
					Matrix matrix = new Matrix();
				
					BitmapFactory.Options options = new BitmapFactory.Options();
					options.inJustDecodeBounds = true;
				 
					if(orientation == 90 || orientation == 270){
						matrix.postRotate(90);
					}else{
						matrix.postRotate(0);
					}

					Bitmap thumbnail = Bitmap.createScaledBitmap(selected, THUMBNAIL_SIZE, THUMBNAIL_SIZE, false);
					Bitmap rotated = Bitmap.createBitmap(thumbnail, 0, 0, thumbnail.getHeight(), thumbnail.getWidth(), matrix, true);
					pictureMessageList.add(rotated);
				}else{
					pictureAddresses.add(null);
					pictureMessageList.add(null);
				}
				
				
				
				//byte[] pictureAsBytes = Base64.decode(content[5], (Base64.DEFAULT));
				//Bitmap bp = BitmapFactory.decodeByteArray(pictureAsBytes, 0, pictureAsBytes.length);
				//pictureMessageList.add(bp);
				
				 //ArrayList<Bitmap> pictureMessageList
				//boolean read, Bitmap bmp, Uri uri
	    	 }
	    	 br.close();
	    	 }catch(IOException e){
	    		 e.printStackTrace();
	    		 
	    	 }
	    	 
	    }
	
	/**
	 * Method to read in a text file placed in the res/raw directory of the
     * application. The method reads in all lines of the file sequentially.
	 * @param categoryList List with different categories
	 * @param contentLists List with different lists of category content
	 */
	public void readRaw(ArrayList<String> categoryList, ArrayList<ArrayList<String>> contentLists) {
		InputStream is = context.getResources().openRawResource(R.raw.saved_content);
		InputStreamReader isr = new InputStreamReader(is);
		/* Second argument is buffer-size */
		BufferedReader br = new BufferedReader(isr, 8192);  
    
		try {
			String test;
			while (true) {
				test = br.readLine();
				int count_number = 0; 
				// readLine() returns null if no more lines in the file
				if (test==(null)) break;
				String[] content = test.split("_");
				for(int j =0; j< content.length; j++){ 
					if(Character.isDigit(content[j].charAt(0))){
						count_number = count_number +1;
						}
					}
				if(count_number == 1){
					int position = Integer.parseInt(content[0]);
					
					categoryList.add((position), content[1]);
					contentLists.add((position), new ArrayList<String>());
					contentLists.get(position).add("Add new content");
					}
				if(count_number == 2){
					int categoryPosition = Integer.parseInt(content[0]);
					int size = contentLists.get(categoryPosition).size();
					contentLists.get(categoryPosition).add((size-1), content[2]);
				}
				}
            	isr.close();
            	is.close();
            	br.close();
        	} catch (IOException e) {
        		e.printStackTrace();
        	}
    	}
	
	/**
	 * Reads the size of the file in the raw-folder
	 */
	public int sizeOfRaw(){
		InputStream is = context.getResources().openRawResource(R.raw.saved_content);
		int size = 0;
		try{
			size = is.available(); 
		}catch(IOException e){
			e.printStackTrace();
		}
		return size; 
	}
	/**
	 * Returns the number of lines in file in raw-folder
	 */
	public int numberOfLinesRaw(){
		InputStream is = context.getResources().openRawResource(R.raw.saved_content);
		InputStreamReader isr = new InputStreamReader(is);
		BufferedReader br = new BufferedReader(isr, 8192);
		int countLines = 0;
    
		try {
			String test;
			while (true) {
				test = br.readLine(); 
				countLines++; 
				// readLine() returns null if no more lines in the file
				if (test==(null)) break;
			}
		}catch (IOException e) {
			e.printStackTrace();
		}
		return countLines;
	}
	/**
	 * Reads the size of the file in the SD-directory
	 * @param filename name of file to be read
	 */
	public long sizeOfSDFile(String filename){
		File root = android.os.Environment.getExternalStorageDirectory();
		File dir = new File(root.getAbsolutePath() + "/download");
		dir.mkdirs();
		File file = new File(dir, filename);
	
		long size = file.length();
		return size; 
	}
	/**
	 * Deletes file in SD-director
	 * @param filename name if file to be deleted
	 */
	public void deleteSDFile(String filename){
		File root = android.os.Environment.getExternalStorageDirectory();
		File dir = new File(root.getAbsolutePath() + "/download");
		dir.mkdirs();
		File file = new File(dir, filename);
		file.delete();
	}

}
